<?php
/*
Plugin Name: Blogroll Links Favicons
Plugin URI: http://johndturner.com/wordpress-stuff/plugins/blogroll-links-favicons/
Description: Add favicons on your blogroll links. Caches links so it does not have to go out each time and find the favicons. [<a href="link-manager.php?page=blogroll-links-favicons.php">Settings</a>]
Author: John Turner
Version: 1.0
Author URI:  http://johndturner.com
*/

// Add Option
add_option('blogroll_favicons', array());

// Main Function
function blogroll_favicons($output){
	$bookmark_arr = explode("\n",$output);
	$cached_favicons = get_option('blogroll_favicons');
	$default_favicon = get_option('default_favicon');
	$cache_update = array();
	foreach($bookmark_arr as $k => $v){
		if(preg_match('/href="(.*)"/', $v, $matches) && !preg_match('/img/', $v)){
			preg_match('/http:\/\/[a-zA-Z0-9-\.]+(\/|\s|$|\")/', $matches[1],$urls);
			preg_match('/http:\/\/[a-zA-Z0-9-\.\/]+(\s|$|\")/', $matches[1],$fullurls);
			$bookmark_url = str_replace('"', "", $urls[0]);
			$bookmark_fullurl = str_replace('"', "", $fullurls[0]);
            if(!empty($cached_favicons[$bookmark_url])){
				if($cached_favicons[$bookmark_url] == 'empty'){
					$favicon = '';
					$cache_update[$bookmark_url] = 'empty';	
				} else {
				  	$favicon = $cached_favicons[$bookmark_url];
					$cache_update[$bookmark_url] = $favicon;	
				}
			} else {
				$favicon = noio_locate_icon($bookmark_url);
				if(empty($favicon)) {
					$cache_update[$bookmark_url] = 'empty';
				} else {
					$cache_update[$bookmark_url] = $favicon;
				}
				
			}
			$str = explode('<li>', $v);
			$new_bookmark = '<li>';
			if(!empty($favicon) || !empty($default_favicon)){
				if(!empty($favicon)) {
					$new_bookmark .= '<a href="'.$bookmark_fullurl.'"><img style="vertical-align: text-top;width:16px;height:16px" class="blogroll-favicon" src="'.$favicon.'" alt="" /></a> ';
				} else if (!empty($default_favicon)) {
					$new_bookmark .= '<a href="'.$bookmark_fullurl.'"><img style="vertical-align: text-top;width:16px;height:16px" class="blogroll-favicon" src="'.$default_favicon.'" alt="" /></a> ';
				}
	     	}
			$new_bookmark .= $str[1];
			$bookmark_arr[$k] = $new_bookmark;
		}
	}
	update_option('blogroll_favicons',$cache_update);
	$output = implode("\n",$bookmark_arr);
	return $output;
}


// Add Filter
add_filter('wp_list_bookmarks', 'blogroll_favicons');

// Add option page

function blogroll_favicons_options(){
	$msg ='';
	$nonce=$_REQUEST['_wpnonce'];
	
	if (wp_verify_nonce($nonce, 'my-nonce') ) {
		if($_REQUEST['bi_clear_cache'] == 'true'){
			update_option('blogroll_favicons',array());
		}
		update_option('default_favicon', $_REQUEST['bi_default_favicon'] );
		$msg .='<div class="updated fade below-h2" id="message"><p>Updated</p></div>';
	}
	$nonce= wp_create_nonce('my-nonce');
	$default_favicon = get_option('default_favicon');
	/*if( ini_get('allow_url_fopen') ){
	    $msg .='';
	}else{
	    $msg .='<div class="error" id="error"><p>Please make sure \'allow_url_fopen = On\' is set in your php.ini file.</p></div>';
	}*/
	print('
			<div class="wrap">
				<h2>'.__('Blogroll Links Favicons Options ', 'blogrollfavicons').'</h2>'.$msg.'
				<form id="blogrollfavicons" name="blogrollfavicons" method="post">
				<table class="form-table">
				<tr>
					<td colspan="2" >
						<p><strong>The 1st time the page with the blogroll loads, it could be slow, after the favicons are cached page speed will be normal again.</strong></p>
					</td>
				</tr>
				<tr valign="top"> 
					<th scope="row">Default Favicon URL:
			    	</th>
			    	<td><input type="text" name="bi_default_favicon" value="'.$default_favicon.'" /> (If the favicon on the link is missing this will be shown instead.)
					</td>
				</tr>
				<tr valign="top"> 
					<th scope="row">Clear Cache:
			    	</th>
			    	<td><input type="checkbox" name="bi_clear_cache" value="true" />
					</td>
				</tr>
				<tr valign="top"> 
					<td colspan="2">
						<p class="submit">
							<input type="submit" name="submit_button" value="Update" />
						</p>
						<input type="hidden" name="_wpnonce" value="'.$nonce.'" />
					</td>
				</tr>		
				</table>
				</form>
			</div>
		 ');
	
}



function blogroll_favicons_items() {
	add_submenu_page(
		'link-manager.php',
		__('Link Favicons Options ', 'blogrollfavicons')
		, __('Link Favicons', 'blogrollfavicons')
		, 8 
		, basename(__FILE__)
		, 'blogroll_favicons_options'
	);
}

add_action('admin_menu', 'blogroll_favicons_items');

// Functions
function noio_locate_icon($url) {
	require_once( ABSPATH . 'wp-includes/class-snoopy.php');
	$snoopy = new Snoopy();
	$result = $snoopy->fetch($url);
    $html = $snoopy->results;
	if( $result ) {

		if (preg_match('/<link[^>]+rel="(?:shortcut )?icon"[^>]+?href="([^"]+?)"/si', $html, $matches)) {

			$linkUrl = html_entity_decode($matches[1]);
			if (substr($linkUrl, 0, 1) == '/') {
				$urlParts = parse_url($url);
				$faviconURL = $urlParts['scheme'].'://'.$urlParts['host'].$linkUrl;
			} else if (substr($linkUrl, 0, 7) == 'http://') {
				$faviconURL = $linkUrl;
			} else if (substr($url, -1, 1) == '/') {
				$faviconURL = $url.$linkUrl;
			} else {
				$faviconURL = $url.'/'.$linkUrl;
			}

		} else {

			$urlParts = parse_url($url);
			$faviconURL = $urlParts['scheme'].'://'.$urlParts['host'].'/favicon.ico';

		}
		if( $faviconURL_exists = noio_url_validate($faviconURL) )
			return $faviconURL;
	} 
	return false;
}

function noio_url_validate( $link ) {
   
	$url_parts = @parse_url( $link );

	if ( empty( $url_parts["host"] ) )
		return false;

	if ( !empty( $url_parts["path"] ) ) {
		$documentpath = $url_parts["path"];
	} else {
		$documentpath = "/";
	}

	if ( !empty( $url_parts["query"] ) )
		$documentpath .= "?" . $url_parts["query"];

	$host = $url_parts["host"];
	$port = $url_parts["port"];
	
	if ( empty($port) )
		$port = "80";

	$socket = @fsockopen( $host, $port, $errno, $errstr, 30 );
	
	if ( !$socket )
		return false;
		
	fwrite ($socket, "HEAD ".$documentpath." HTTP/1.0\r\nHost: $host\r\n\r\n");

	$http_response = fgets( $socket, 22 );

	$responses = "/(200 OK)|(200 ok)|(30[0-9] Moved)/";
	if ( preg_match($responses, $http_response) ) {
		fclose($socket);
		return true;
	} else {
		
		return false;
	}
}
?>
